/* MOUSE.INC   Must #includ after the dos.h #include to defint the
   register set used to pass args to device driver    */

#define callMDD int86(0x33, &inreg, &outreg)

/* define sorting macros used locally  */
#define lower (x,y) (x < y) ? x : y
#define upper (x,y_ (x > y) ? x : y

/* static registers used throughout  */
union REGS inreg,outreg;

/* Structures used by these functions  */
typedef struct {
	int exists,
		nButtons;
}  resetRec;

typedef struct {
	int ButtonStatus,
		opcount,
		column,row;
}  locRec;

typedef struct {
	int hCount,
	vCount;
}	moveRec;


/*  The following are implementations of the Microsoft Mouse functions */


/*	Resets mouse to default state.  Returns pointer to a structure			*/
/*	indicating whether or not mouse is installed and, if so, how many    */
/*	buttons it has.                                                      */
/*	ALWAYS call this function during program initialization.             */
resetRec *mReset() {
	static resetRec m;
	inreg.x.ax = 0;
	callMDD;
	m.exists = outreg.x.ax;
	m.nButtons =outreg.x.bx;
	return(&m);
}

/*	Makes the mouse cursor visable. Don't call if cursor is already		*/
/* visible, and alternate with calls to mHide.									*/
void mShow(void) {
	inreg.x.ax = 1;
	callMDD;
}

/* Makes mouse cursor invisible. Movement and button activity are still	*/
/*	tracked. Do not call if cursor is already hidden, and alternate with	*/
/* calls to mShow																			*/
void mHide(void) {
	inreg.x.ax = 2;
	callMDD;
}

/* Gets mouse cursor position and button status, returns pointer to		*/
/* structure containing this info													*/
locRec *mPos (void) {
	static locRec m;
	inreg.x.ax = 3;
	callMDD;
	m.buttonStatus = outreg.x.bx;
	m.column = outreg.x.cx;
	m.row = outreg.x.dx;
	return(&m);
}

/* Move mouse cursor to new position */
void mMoveto(int newCol, int newRow) {
	inreg.x.ax = 4;
	inreg.x.cx = newCol;
	inreg.x.dx = newRow;
	callMDD;
}

/* Gets pressed info about named button: current status (up/down),		*/
/* times pressed since last call, position at most recent press.			*/
/* Resets count and position info.  Button 0 is left, 1 is on right, 	*/
/* Returns pointer to locRec structure containing info.						*/
locRec *mPressed(int button) {
	static locRec m;
	inreg.x.ax = 5;
	inreg.x.bx = button;
	callMDD;
	m.buttonStatus = outreg.x.ax;
	m.opCount = outreg.x.bx;
	m.column = outreg.x.cx;
	m.row = outreg.x.dx;
	return(&m);
}

/* Same as mPressed except with release info									*/
locRec *mReleased(int button) {
	static locRec m;
	inreg.x.ax = 6;
	inreg.x.bx = button;
	callMDD;
	m.buttonStatus = outreg.x.ax;
	m.opCount = outreg.x.bx;
	m.column = outreg.x.cx;
	m.row = outreg.x.dx;
	return(&m);
}

/* Sets min and max horozontal range for mouse cursor.  Moves cursor	*/
/* inside range if outside when called.  Swaps values if hmin and		*/
/* hmax are reversed																	*/
void mColRange(int hmin,int hmax) {
	inreg.x.ax = 7;
	inreg.x.cx = hmin;
	inreg.x.dx = hmax;
	callMDD;
}

/* Sets min and max vertical range for mouse cursor same as mColRange*/
void mRowRange(int vmin, int vmax) {
	inreg.x.ax = 8;
	inreg.x.cx = vmin;
	inreg.x.dx = vmax;
	callMDD;
}

/* Sets graphic cursor shape */
void mGraphCursor(int hHot, int vHot, unsigned maskSeg, unsigned maskOfs) {
	struct SREGS seg;
	inreg.x.ax = 9;
	inreg.x.bx = hHot;
	inreg.x.cx = vHot;
	inreg.x.dx = maskOfs;
	seg.es = maskSeg;
	int86x (0x33,&inreg,&outreg,&seg);
}

/* Sets text cursor type, where 0 = software and 1 = hardware				*/
/* For software cursor, arg1 and arg2 are the screen and cursor masks.	*/
/* for hardware cursor, arg1 and arg2 specify scan line start/stop		*/
/* i.e. cursor shape.																	*/
void mTextCursor(int curstype, unsigned arg1, unsigned arg2) {
	inreg.x.ax = 10;
	inreg.x.bx = curstype;
	inreg.x.cx = arg1;
	inreg.x.dx = arg2;
	callMDD;
}

/* Reports net motion of cursor since last call to this function 			*/
moveRec *mMotion(void) {
	static moveRec m;
	inreg.x.ax = 11;
	callMDD;
	m.hCount = _CX;
	m.vCount = _DX;
	return(&m);
}

/* Installs a user-defined task to be executed upon one or more 			*/
/* mouse events specified by mask.													*/
void mInstTask(unsigned mask, unsigned taskSeg, unsigned taskOfs) {
	struct SREGS seg;
	inreg.x.ax = 12;
	inreg.x.cx = mask;
	inreg.x.dx = taskOfs;
	seg.es = taskSeg;
	int86x (0x33, &inreg, &outreg, &seg);
}

/* Turns on light pen emulation. This is the default condition.			*/
void mLpenOn(void) {
	inref.x.ax = 13;
	callMDD;
}

/* Turns off light pen emulation.													*/
void mLpenOff(void) {
	inreg.x.ax = 14;
	callMDD;
}

/* Sets micky to pixel ratio, where ratio is R/8. Default is 16			*/
/* for vertical and 8 for horizontal 												*/
void mRatio(int horiz, int vert) {
	inreg.x.ax = 15;
	inreg.x.cx = horiz;
	inreg.x.dx = vert;
	callMDD;
}

